@@ -39,41 +39,18 @@ char *mimetype;
 #define xalloc malloc
 #define xfree free
 
-static void my_kill(int pid)
-{
-  if(!kill(pid, SIGTERM))
-  {
-    /* we have failed to kill our children twice,
-     * sleep on it and try again tomorrow..
-     */
-    D("killing %d\n",pid);
-    usleep(4000);
-    if(!kill(pid, SIGTERM))
-    {
-      usleep(4000);
-      if(!kill(pid, SIGTERM))
-      {
-	/* We have failed to kill our children a third time -
-	 * wield nukes and try again.
-	 */
-	kill(pid, SIGKILL);
-      }
-    }
-  }
-}
-
 char *my_strstr(char *s1,const char *s2)
 {
-  for(;*s1;s1++)
+  for (;*s1;s1++)
   {
     const char *p1,*p2;
-    for(p1=s1,p2=s2;*p2;p1++,p2++) if(*p2!=*p1) break;
-    if(!*p2) return s1;
+    for (p1=s1,p2=s2;*p2;p1++,p2++)
+      if (*p2!=*p1) break;
+    if (!*p2) return s1;
   }
   return NULL;
 }
 
-
 /*
  * Functions below are called by the child process
  */
@@ -83,30 +60,11 @@ static int childpid=-1;
 void kill_child(int tmp)
 {
   D("GOT SIGTERM\n");
-  if(childpid != -1)
-    my_kill(childpid);
+  if (childpid != -1)
+    kill(childpid, SIGTERM);
   exit(0);
 }
 
-
-static void very_low_run(char **argv)
-{
-#ifdef H_NOISY
-  if(flags & H_NOISY)
-  {
-    int nl=open("/dev/null", O_RDONLY);
-    D("Redirecting stdout and stderr\n");
-    dup2(nl,1);
-    dup2(nl,2);
-    close(nl);
-  }
-#endif
-  
-  execvp(argv[0],argv);
-  D("Execvp failed..%d\n",errno);
-  exit(EX_UNAVAILABLE);
-}
-
 int error_handler(Display *dpy, XErrorEvent *err)
 {
   D("!!!ERROR_HANDLER!!!\n");
@@ -119,7 +77,7 @@ static Window ignored[MAX_IGNORED];
 
 int inital_win_cb(Window w, int depth)
 {
-  if(windows_found < MAX_IGNORED)
+  if (windows_found < MAX_IGNORED)
     ignored[windows_found++]=w;
   return 0;
 }
@@ -127,10 +85,9 @@ int inital_win_cb(Window w, int depth)
 int win_cb(Window w, int depth)
 {
   int q;
-  for(q=0;q<windows_found;q++)
-    if(ignored[q]==w)
+  for (q=0;q<windows_found;q++)
+    if (ignored[q]==w)
       return 0;
-
   return 1;
 }
 
@@ -142,10 +99,11 @@ static Window winrecur(Window from,
   Window root, parent;
   Window *children=NULL;
   int e,num_children;
-
+  Window found=-1;
+  
   D("Winrecur, checking window %x (depth=%d)\n",from,depth);
 
-  if(!XQueryTree(display, from, &root, &parent, &children, &num_children))
+  if (!XQueryTree(display, from, &root, &parent, &children, &num_children))
   {
     D("Querytree failed!!!\n");
     return (Window)0;
@@ -163,9 +121,9 @@ static Window winrecur(Window from,
     XWindowAttributes a;
 
     D("XGetWindowAttributes\n");
-    if(!XGetWindowAttributes(display, children[e], &a))
+    if (!XGetWindowAttributes(display, children[e], &a))
       continue;
-    if(a.map_state != IsViewable) continue;
+    if (a.map_state != IsViewable) continue;
 
     D("XFetchName\n");
     if (0 != XFetchName(display, children[e], &windowname))
@@ -176,9 +134,12 @@ static Window winrecur(Window from,
       if (!name || !strcmp(windowname, name))
 	yes=cb(children[e],depth);
       XFree(windowname);
-      if(yes) return children[e];
+      if (yes) found = children[e];
     }
 
+    /* Return the last window found */
+    if (found != -1) return found;
+
     D("XGetTextProperty\n");
     property=0;
     XGetWindowProperty(display, children[e], XA_WM_CLASS,
@@ -188,9 +149,9 @@ static Window winrecur(Window from,
 		       &type, &fmt, &nitems, &bytes,
 		       &property);
 
-    if(property)
+    if (property)
     {
-      if(type == XA_STRING && fmt==8)
+      if (type == XA_STRING && fmt==8)
       {
 	int len=nitems;
 	char *n=(char *)property;
@@ -198,8 +159,8 @@ static Window winrecur(Window from,
 	{
 	  D("Winrecur, checking window CLASS %x (%s != %s)\n",children[e],n,name);
 	  if (!name || !strcmp(n, name))
-	    if(cb(children[e],depth))
-	      return children[e];
+	    if (cb(children[e],depth))
+	      found= children[e];
 	  
 	  len-=strlen(n)+1;
 	  n+=strlen(n)+1;
@@ -208,14 +169,16 @@ static Window winrecur(Window from,
       XFree(property);
     }
   }
-  if(depth>0)
+  if (found != -1) return found;
+  if (depth>0)
   {
     for (e=0;e<num_children;e++)
     {
       Window ret=winrecur(children[e], depth-1, name, cb);
-      if(ret) return ret;
+      if (ret) found = ret;
     }
   }
+  if (found != -1) return found;
   XFree(children);
   return (Window)0;
 }
@@ -242,17 +205,17 @@ static void place_window(void)
   int w=windata.width;
   int h=windata.height;
 
-  if(old_parent != parent)
+  if (old_parent != parent)
   {
     XWithdrawWindow(display, victim,
 		    XScreenNumberOfScreen (wattr.screen));
     XSync (display, FALSE);
 
-    if(!old_parent)
+    if (!old_parent)
     {
       XWMHints *leader_change;
       D("Changing leader of window %x\n",victim);
-      if((leader_change = XGetWMHints(display, victim)))
+      if ((leader_change = XGetWMHints(display, victim)))
       {
 	leader_change->flags = (leader_change->flags | WindowGroupHint);
 	leader_change->window_group = wattr.root;
@@ -262,7 +225,7 @@ static void place_window(void)
       }
     }
 
-    for(x=0;x<25;x++)
+    for (x=0;x<25;x++)
     {
       D("Reparenting window %x into %x\n",victim,WINDOW);
       XReparentWindow (display, victim, WINDOW, 0, 0);
@@ -270,7 +233,7 @@ static void place_window(void)
     }
   }
 
-  if(flags & H_FILL)
+  if (flags & H_FILL)
   {
     D("Resizing window %x with FILL\n",victim);
     D("New size: %dx%d+%d+%d (of %dx%d)\n",w,h,
@@ -281,15 +244,15 @@ static void place_window(void)
 		      0,0,
 		      w,h);
   }
-  else if(flags & H_MAXASPECT)
+  else if (flags & H_MAXASPECT)
   {
     int tmpw, tmph;
     D("Resizing window %x with MAXASPECT\n",victim);
-    if(xaspect && yaspect)
+    if (xaspect && yaspect)
     {
       tmph=h/yaspect;
       tmpw=w/xaspect;
-      if(tmpw<tmph) tmph=tmpw;
+      if (tmpw<tmph) tmph=tmpw;
       tmpw=tmph*xaspect;
       tmph=tmph*yaspect;
       
@@ -344,11 +307,11 @@ static void my_xlockwindow(Display *dpy,
 			  XA_STRING,
 			  &type, &fmt, &nitems, &bytes,
 			  &property);
-      if(!property) break;
+      if (!property) break;
       D("Property already set, bytes_left=%d, waiting..\n",bytes);
       XFree(property);
       gettimeofday(&tv2, 0);
-      if(tv2.tv_sec > tv.tv_sec)
+      if (tv2.tv_sec > tv.tv_sec)
       {
 	D("Stealing lock....\n");
 	/* Time to steal the lock! */
@@ -372,17 +335,16 @@ static void my_xlockwindow(Display *dpy,
 		       XA_STRING,
 		       &type, &fmt, &nitems, &bytes,
 		       &property);
-    if(property)
+    if (property)
     {
       int done=!strncmp(buffer,property,strlen(buffer));
       D("LOCK: done=%d, bytes_left=%d\n",done,bytes);
       XFree(property);
-      if(done) return;
+      if (done) return;
     }
   }
 }
 
-
 static void my_xunlockwindow(Display *dpy,
 			     Window win,
 			     Atom prop)
@@ -400,12 +362,172 @@ static void sigwinch(int tmp)
   got_sigwinch=1;
 }
 
+static void setup_swallow()
+{
+  D("Setting up X data (%s)\n",displayname);
+
+  if (!display)
+    {
+      display = XOpenDisplay(displayname);
+      XSetErrorHandler(error_handler);
+    }
+  D("display=%x\n",display);
+
+  if (display && XGetWindowAttributes(display, WINDOW, &wattr))
+    {
+      victim=0;
+      old_parent=0;
+      xaspect=yaspect=0;
+      D("rootwin=%x\n",wattr.root);
+      lock_property=XInternAtom(display, "PLUGGER_SWALLOW_LOCK", 0);
+      D("lock_property=%x\n",lock_property);
+      
+      my_xlockwindow(display, wattr.root, lock_property);
+      
+      windows_found=0;
+      winrecur(wattr.root, 3, winname, inital_win_cb);
+      
+      D("Done Setting up X data\n");
+    }
+  else
+    {
+      D("XGetWIndowAttributes failed!!!!\n");
+      flags &=~ H_SWALLOW;
+    }
+}
+
+static void run_app(char **argv)
+{
+#ifdef H_NOISY
+  int nl;
+  if (flags & H_NOISY)
+  {
+    D("Redirecting stdout and stderr\n");
+    if ((nl = open("/dev/null", O_RDONLY)) == -1)
+      exit(EX_UNAVAILABLE);
+    dup2(nl,1);
+    dup2(nl,2);
+    close(nl);
+  }
+#endif
+  
+  execvp(argv[0],argv);
+  D("Execvp failed..%d\n",errno);
+  exit(EX_UNAVAILABLE);
+}
+
+static void handle_app(int pid)
+{
+  int waitpid_done=0;
+  int unlock_done=0;
+  int status;
+
+  if (!(flags & H_DAEMON))
+    childpid=pid;
+  
+  if (flags & H_SWALLOW)
+    {
+      int tmp;
+      for (tmp=0; tmp<500; tmp++) /* approx 10 seconds */
+	{
+	  if (!(flags & H_EXITS) && waitpid(pid, &status, WNOHANG) > 0)
+	    {
+	      waitpid_done=1;
+	      break;
+	    }
+	  
+	  D("Looking for victim... (%s)\n",winname);
+	  
+	  if ((victim=winrecur(wattr.root, 3, winname, win_cb)))
+	    {
+	      my_xunlockwindow(display, wattr.root, lock_property);
+	      unlock_done=1;
+	      
+	      D("Found it!! Victim=%x\n",victim);
+	      if (flags & H_MAXASPECT)
+		{
+		  int tmp;
+		  XWindowAttributes ca;
+		  XGetWindowAttributes(display, victim, &ca);
+		  while (1)
+		    {
+		      tmp=gcd(ca.width, ca.height);
+		      xaspect=ca.width/tmp;
+		      yaspect=ca.height/tmp;
+		      if (xaspect>1 && yaspect>1 && (xaspect+yaspect>100))
+			{
+			  xaspect>>=1;
+			  xaspect>>=1;
+			} else
+			  break;
+		    }
+		  D("xaspect=%d yaspect=%d\n",xaspect,yaspect);
+		}
+
+	      place_window();
+	      break;
+	    }
+
+	  usleep(2000); /* 1/50 second */
+	}
+      if (!unlock_done)
+	{
+	  my_xunlockwindow(display, wattr.root, lock_property);
+	  unlock_done=1;
+	}
+    }
+  
+  while (!waitpid_done)
+    {
+      if (got_sigwinch)
+	{
+	  Window oldwindow=WINDOW;
+	  got_sigwinch=0;
+	  
+	  D("Got SIGWINCH, new parent=%x\n",WINDOW);
+	  
+	  if (victim)
+	    {
+	      place_window();
+	    } else {
+	      if (my_strstr(command,"$window") &&
+		  oldwindow != WINDOW)
+		{
+		  kill(pid, SIGKILL);
+		  waitpid_done++;
+		  continue;
+	      }
+	    }
+	}
+      D("Waiting for godot... %d\n",pid);
+      if (flags & H_EXITS)
+	{
+	  exit(0);
+	} else {
+	  if (waitpid(pid,&status,0) > 0)
+	    waitpid_done++;
+	}
+      D("Waid done?? (%d)\n",waitpid_done);
+    }
+  
+  if (!WIFEXITED(status))
+    {
+      D("Process dumped core or something...\n");
+      exit(10);
+    }
+  if (WEXITSTATUS(status) && !(flags & H_IGNORE_ERRORS))
+    {
+      D("Process exited with error code: %d\n",WEXITSTATUS(status));
+      exit(WEXITSTATUS(status));
+    }
+  D("exited ok!\n");
+}
 
 int main(int argc, char **argv)
 {
   D("Helper started.....\n");
 
-  if(argc < 7)
+  if (argc < 7)
   {
     fprintf(stderr,"MozPlugger version " VERSION " helper application.\n"
 	    "Written by Fredrik Hubinette and Louis Bavoil.\n" 
@@ -430,50 +552,50 @@ int main(int argc, char **argv)
   command=argv[5];
   mimetype=argv[6];
 
-  if(!strlen(file)) file=0;
-  if(!strlen(winname)) winname=0;
-  if(!strlen(displayname)) displayname=0;
+  if (!strlen(file)) file=0;
+  if (!strlen(winname)) winname=0;
+  if (!strlen(displayname)) displayname=0;
 
-  D("HELPER: %s %s %s %s %s %s\n",
+  D("HELPER: %s %s %s %s %s\n",
     argv[0],
     argv[1],
     file,
-    winname,
     displayname,
     command);
 
-  while(repeats>0)
+  while (repeats > 0)
   {
     char *argv[10];
     char buffer[16384];
     char *foo=buffer;
     int loops=1;
+    int pid;
 
-    /* Hmm, what does H_REPEATCOUNT do again?? */
-    if(flags & H_REPEATCOUNT)
+    /* This application will use the $repeat variable */
+    if (flags & H_REPEATCOUNT)
     {
       loops=repeats;
     }
 
     /* This application will play the data forever */
-    if(flags & H_LOOP)
+    if (flags & H_LOOP)
     {
       D("Expecting application to loop.\n");
       loops=0x7fffffff;
     }
 
     /* setup environment variable $file */
-    if((flags & H_MANY) && !(flags & H_REPEATCOUNT))
+    if ((flags & H_MANY) && !(flags & H_REPEATCOUNT))
       {
 	int e;
 	sprintf(foo,"file=%s",file);
 	loops=MIN(repeats,10);
-	for(e=0;e<loops;e++)
+	for (e=0;e<loops;e++)
 	  {
 	    strcat(foo," ");
 	    strcat(foo,file);
 	  }
-      }else{
+      } else {
 	sprintf(foo,"file=%s",file);
       }
     putenv(foo);
@@ -501,7 +623,7 @@ int main(int argc, char **argv)
     foo+=strlen(foo)+1;
 
     /* setup environment variable $DISPLAY */
-    if(displayname)
+    if (displayname)
     {
       sprintf(foo,"DISPLAY=%s",displayname);
       D("putenv(%s)\n",foo);
@@ -510,7 +632,7 @@ int main(int argc, char **argv)
     }
 
     /* setup environment variable $mimetype */
-    if(mimetype)
+    if (mimetype)
     {
       sprintf(foo,"mimetype=%s",mimetype);
       D("putenv(%s)\n",foo);
@@ -524,164 +646,26 @@ int main(int argc, char **argv)
     argv[2]=command;
     argv[3]=0;
 
-
     D("Execing %s (repeats=%d loops=%d)\n",command,repeats,loops);
 
-    if(flags & H_SWALLOW)
-    {
-      D("Setting up X data (%s)\n",displayname);
-      if(!display)
-      {
-	display = XOpenDisplay(displayname);
-	XSetErrorHandler(error_handler);
-      }
-      D("display=%x\n",display);
-      if(display &&
-	 XGetWindowAttributes(display, WINDOW, &wattr))
-      {
-	victim=0;
-	old_parent=0;
-	xaspect=yaspect=0;
-	D("rootwin=%x\n",wattr.root);
-	lock_property=XInternAtom(display, "PLUGGER_SWALLOW_LOCK", 0);
-	D("lock_property=%x\n",lock_property);
-
-	my_xlockwindow(display, wattr.root, lock_property);
-
-	windows_found=0;
-	winrecur(wattr.root, 3, winname, inital_win_cb);
-
-	D("Done Setting up X data\n");
-      }else{
-	D("XGetWIndowAttributes failed!!!!\n");
-	flags &=~ H_SWALLOW;
-      }
-    }
+    if (flags & H_SWALLOW)
+      setup_swallow();
     
-    {
-      int pid;
-      D("Running %s\n",command);
-      pid=fork();
-      if(pid==-1) exit(10);
+    D("Running %s\n",command);
+
+    if ((pid = fork()) == -1)
+      exit(10);
       
-      if(!pid)
+    if (!pid)
       {
-	very_low_run(argv);
-      }else{
-	int waitpid_done=0;
-	int unlock_done=0;
-	int status;
-	if(!(flags & H_DAEMON))
-	  childpid=pid;
-	D("waiting for (%d)\n",pid);
-
-	if(flags & H_SWALLOW)
-	{
-	  int tmp;
-	  for(tmp=0;tmp<500;tmp++) /* approx 10 seconds */
-	  {
-	    if(!(flags & H_EXITS))
-	    {
-	      if(waitpid(pid, &status, WNOHANG) > 0)
-	      {
-		waitpid_done=1;
-		break;
-	      }
-	    }
-
-	    D("Looking for victim... (%s)\n",winname);
-	    if((victim=winrecur(wattr.root, 3, winname, win_cb)))
-	    {
-	      my_xunlockwindow(display, wattr.root, lock_property);
-	      unlock_done=1;
-
-	      D("Found it!! Victim=%x\n",victim);
-	      if(flags & H_MAXASPECT)
-	      {
-		int tmp;
-		XWindowAttributes ca;
-		XGetWindowAttributes(display, victim, &ca);
-		while(1)
-		{
-		  tmp=gcd(ca.width, ca.height);
-		  xaspect=ca.width/tmp;
-		  yaspect=ca.height/tmp;
-		  if(xaspect>1 && yaspect>1 && (xaspect+yaspect>100))
-		  {
-		    xaspect>>=1;
-		    xaspect>>=1;
-		  }else
-		    break;
-		}
-		D("xaspect=%d yaspect=%d\n",xaspect,yaspect);
-	      }
-
-	      place_window();
-	      break;
-	    }
-
-	    usleep(2000); /* 1/50 second */
-	  }
-	  if(!unlock_done)
-	  {
-	    my_xunlockwindow(display, wattr.root, lock_property);
-	    unlock_done=1;
-	  }
-	}
-
-	while(!waitpid_done)
-	{
-	  if(got_sigwinch)
-	  {
-	    Window oldwindow=WINDOW;
-	    got_sigwinch=0;
-
-	    D("Got SIGWINCH, new parent=%x\n",WINDOW);
-
-	    if(victim)
-	    {
-	      place_window();
-	    }else{
-	      if(my_strstr(command,"$window") &&
-		 oldwindow != WINDOW)
-	      {
-		/*
-		 * Kill and re-start child
-		 * Hmm, what happens if we are streaming????
-		 */
-		my_kill(pid);
-		waitpid_done++;
-		continue;
-	      }
-	    }
-	  }
-	  D("Waiting for godot... %d\n",pid);
-	  if(flags & H_EXITS)
-	  {
-	    exit(0);
-	  }else{
-	    if(waitpid(pid,&status,0) > 0)
-	      waitpid_done++;
-	  }
-	  D("Waid done?? (%d)\n",waitpid_done);
-	}
-
-	D("wait done repeats=%d loops=%d\n",repeats, loops);
-	if(!WIFEXITED(status))
-	{
-	  D("Process dumped core or something...\n");
-	  exit(10);
-	}
-	if(WEXITSTATUS(status) && !(flags & H_IGNORE_ERRORS))
-	{
-	  D("Process exited with error code: %d\n",WEXITSTATUS(status));
-	  exit(WEXITSTATUS(status));
-	}
-	D("exited ok!\n");
+	run_app(argv);
+      } else {
+	D("waiting for (%d)\n", pid);
+	handle_app(pid);
+	D("wait done repeats=%d loops=%d\n", repeats, loops);
+	if (repeats < MAXINT)
+	  repeats-=loops;
       }
-      if(repeats < MAXINT)
-	repeats-=loops;
-    }
   }
   _exit(0);
 }
