@@ -48,7 +48,6 @@ struct data
 #define xalloc malloc
 #define xfree free
 
-
 /*
  * Helper functions
  */
@@ -62,27 +61,27 @@ static int inpath(char *file)
   char tmp[16384];
   char *path=getenv("PATH");
   char *pos;
-  if(!path) return 0;
+  if (!path) return 0;
 
   D("inpath(%s)\n",file);
-  if(file[0]=='/')
+  if (file[0]=='/')
     return !stat(file, &buf);
 
-  if(!strncmp(file,"internal:",9)) return 1;
+  if (!strncmp(file,"internal:",9)) return 1;
 
   /* Ugly ugly ugly */
-  if(file[0]=='i' && file[1]=='f' && isspace(file[2]))
+  if (file[0]=='i' && file[1]=='f' && isspace(file[2]))
     file+=3;
 
   /* Insert magical shell-script tests here */
 
   D("Hmm? PATH=%s\n",path);
   pos=path;
-  while(1)
+  while (1)
   {
     char *next;
     next=FEND(pos,':');
-    if(next!=pos)
+    if (next!=pos)
     {
       char *ptr=tmp;
 
@@ -95,59 +94,55 @@ static int inpath(char *file)
       ptr+=FEND(file,' ')-file;
       *ptr=0;
       D("stat(%s)\n",tmp);
-      if(!stat(tmp, &buf)) return 1;
+      if (!stat(tmp, &buf)) return 1;
       D("nope\n");
     }
-    if(!*next) return 0;
+    if (!*next) return 0;
     pos=next+1;
   }
-  D("GAAHHH!\n");
 }
 
-#ifndef MAX_FD
-#define MAX_FD 1024
-#endif
-
-static void handler(int sig)
-{
-  int status;
-  wait(&status);
-}
-
-/* Shell for fork which handles some magic needed to prevent
- * netscape from freaking out. It also prevents interferrance
- * from signals.
+/* Shell for fork which handles some magic needed to prevent netscape
+ * from freaking out. It also prevents interferrance from signals.
  */
 
 static int my_fork(NPP instance)
 {
   int pid;
   sigset_t set,oset;
-  sigfillset(& set);
-  D(">>>>>>>>Forking<<<<<<<<,\n");
+
+  /* Mask all the signals to avoid being interrupted by a signal */
+  sigfillset(&set);
   sigprocmask(SIG_SETMASK,&set,&oset);
-  signal(SIGCHLD, &handler);
-  pid=fork();
-  if(pid==-1) return pid;
-  if(!pid)
+
+  D(">>>>>>>>Forking<<<<<<<<,\n");
+
+  if ((pid = fork()) == -1)
+    return -1;
+
+  if (!pid)
   {
     int signum;
     alarm(0);
-    if(!(THIS->flags & H_DAEMON))
+
+    if (!(THIS->flags & H_DAEMON))
       setsid();
 
-    for(signum=0;signum<NSIG;signum++)
+    for (signum=0;signum<NSIG;signum++)
       signal(signum, SIG_DFL);
 
 #ifdef DEBUG
     debug_output=0;
-    D("Chilid re-opened ndebug\n");
+    D("Child re-opened ndebug\n");
 #endif
     THIS->display=0;
-  }else{
+  } else {
     D("Child running with pid=%d\n",pid);
   }
+
+  /* Restore the signal mask */
   sigprocmask(SIG_SETMASK,&oset,&set);
+
   return pid;
 }
 
@@ -165,11 +160,10 @@ static void run(NPP instance, const char *file)
 	  (int) THIS->windata.y,
 	  (int) THIS->windata.width,
 	  (int) THIS->windata.height);
-  D("Executing helper: %s %s %s %s %s %s %s\n",
+  D("Executing helper: %s %s %s %s %s %s\n",
     helper_fname,
     buffer,
     file,
-    THIS->winname,
     THIS->displayname,
     THIS->command,
     THIS->mimetype);
@@ -235,20 +229,20 @@ static void read_config(FILE *f)
 
   handlep=&first_handle;
 
-  while(!feof(f))
+  while (!feof(f))
   {
     fgets(buffer,sizeof(buffer),f);
     D("::: %s",buffer);
-    if(buffer[0]=='#' || !buffer[0] || buffer[0]=='\n') continue;
+    if (buffer[0]=='#' || !buffer[0] || buffer[0]=='\n') continue;
 
-    if(buffer[strlen(buffer)-1]=='\n')
+    if (buffer[strlen(buffer)-1]=='\n')
       buffer[strlen(buffer)-1]=0;
 
-    if(isspace(buffer[0]))
+    if (isspace(buffer[0]))
     {
       char *x=buffer+1;
-      while(isspace(*x)) x++;
-      if(!*x)
+      while (isspace(*x)) x++;
+      if (!*x)
       {
 	D("Empty line.\n");
 	continue;
@@ -257,7 +251,7 @@ static void read_config(FILE *f)
       D("New command\n");
 
       *commandp=(struct command *)xalloc(sizeof(struct command));
-      if(!*commandp)
+      if (!*commandp)
       {
 	D("xalloc failed\n");
 	return;
@@ -269,7 +263,7 @@ static void read_config(FILE *f)
 
       /* Command */
       have_commands++;
-      while(*x!=':' && *x)
+      while (*x!=':' && *x)
       {
 /* 	D("Parsing %s\n",x); */
 	switch(*x)
@@ -282,7 +276,7 @@ static void read_config(FILE *f)
 
 	  default:
 #define GOBBLE(X,Y) \
-  if(gobble(x,X)) { x+=strlen(X); (*commandp)->flags|=Y; break; }
+  if (gobble(x,X)) { x+=strlen(X); (*commandp)->flags|=Y; break; }
 	    GOBBLE("repeat",H_REPEATCOUNT);
 	    GOBBLE("loop",H_LOOP);
 	    GOBBLE("stream",H_STREAM);
@@ -297,20 +291,20 @@ static void read_config(FILE *f)
 #ifdef H_NOISY
 	    GOBBLE("noisy",H_NOISY);
 #endif
-	    if(gobble(x,"swallow"))
+	    if (gobble(x,"swallow"))
 	    {
 	      char *end;
 	      (*commandp)->flags|=H_SWALLOW;
 	      x+=7;
-	      while(isspace(*x)) x++;
-	      if(*x != '(')
+	      while (isspace(*x)) x++;
+	      if (*x != '(')
 	      {
 		fprintf(stderr,"Expected '(' after 'swallow'\n");
 		continue;
 	      }
 	      x++;
 	      end=strchr(x,')');
-	      if(end)
+	      if (end)
 	      {
 		(*commandp)->winname=xalloc(end - x +1);
 		memcpy((*commandp)->winname, x, end-x);
@@ -326,15 +320,15 @@ static void read_config(FILE *f)
 	    x+=strlen(x);
 	}
       }
-      if(*x==':')
+      if (*x==':')
       {
 	x++;
-	while(isspace(*x)) x++;
+	while (isspace(*x)) x++;
 	(*commandp)->cmd=strdup(x);
-      }else{
+      } else {
 	D("No colon? (%s)\n",x);
       }
-      if(!(*commandp)->cmd)
+      if (!(*commandp)->cmd)
       {
 	xfree(*commandp);
 	*commandp=0;
@@ -342,16 +336,16 @@ static void read_config(FILE *f)
 	return;
       }
       commandp=&((*commandp)->next);
-    }else{
+    } else {
       /* Mime type */
 
-      if(have_commands)
+      if (have_commands)
       {
   	D("New handle\n");
-	if(commandp)
+	if (commandp)
 	  D("Commandp=%p\n",*commandp);
 	*handlep=(struct handle *)xalloc(sizeof(struct handle));
-	if(!*handlep)
+	if (!*handlep)
 	  return;
 	
 	(*handlep)->commands=0;
@@ -365,13 +359,13 @@ static void read_config(FILE *f)
 
       D("New mimetype\n");
       *typep=(struct mimetype *)xalloc(sizeof(struct mimetype));
-      if(!*typep)
+      if (!*typep)
 	return;
 
       (*typep)->next=0;
       (*typep)->line=strdup(buffer);
 
-      if(!(*typep)->line)
+      if (!(*typep)->line)
       {
 	xfree(*typep);
 	*typep=0;
@@ -390,37 +384,37 @@ int find_helper_file(char *basename,
   static char fname[8192];
   char *tmp;
   D("find_helper_file '%s'\n",basename);
-  if((tmp=getenv("HOME")) && strlen(tmp)<8000)
+  if ((tmp=getenv("HOME")) && strlen(tmp)<8000)
   {
     sprintf(fname,"%s/.netscape/%s",tmp,basename);
-    if(cb(fname,data)) return 1;
+    if (cb(fname,data)) return 1;
     sprintf(fname,"%s/.mozilla/%s",tmp,basename);
-    if(cb(fname,data)) return 1;
+    if (cb(fname,data)) return 1;
     sprintf(fname,"%s/.opera/%s",tmp,basename);
-    if(cb(fname,data)) return 1;
+    if (cb(fname,data)) return 1;
   }
 
-  if((tmp=getenv("MOZILLA_HOME")) && strlen(tmp)<8000)
+  if ((tmp=getenv("MOZILLA_HOME")) && strlen(tmp)<8000)
   {
     sprintf(fname,"%s/%s",tmp,basename);
-    if(cb(fname, data)) return 1;
+    if (cb(fname, data)) return 1;
   }
 
-  if((tmp=getenv("OPERA_DIR")) && strlen(tmp)<8000)
+  if ((tmp=getenv("OPERA_DIR")) && strlen(tmp)<8000)
   {
     sprintf(fname,"%s/%s",tmp,basename);
-    if(cb(fname, data)) return 1;
+    if (cb(fname, data)) return 1;
   }
 
   sprintf(fname,"/usr/local/netscape/%s",basename);
-  if(cb(fname, data)) return 1;
+  if (cb(fname, data)) return 1;
   sprintf(fname,"/etc/%s",basename);
-  if(cb(fname, data)) return 1;
+  if (cb(fname, data)) return 1;
   sprintf(fname,"/usr/etc/%s",basename);
-  if(cb(fname, data)) return 1;
+  if (cb(fname, data)) return 1;
   sprintf(fname,"/usr/local/etc/%s",basename);
-  if(cb(fname, data)) return 1;
-  if(cb(basename, data)) return 1;
+  if (cb(fname, data)) return 1;
+  if (cb(basename, data)) return 1;
   
   return 0;
 }
@@ -428,7 +422,7 @@ int find_helper_file(char *basename,
 static int read_config_cb(char *fname, void *data)
 {
   FILE *f=fopen(fname,"r");
-  if(!f) return 0;
+  if (!f) return 0;
   read_config(f);
   fclose(f);
   return 1;
@@ -437,7 +431,7 @@ static int read_config_cb(char *fname, void *data)
 static int find_plugger_helper_cb(char *fname, void *data)
 {
   struct stat buf;
-  if(stat(fname, &buf)) return 0;
+  if (stat(fname, &buf)) return 0;
   helper_fname=strdup(fname);
   return 1;
 }
@@ -445,10 +439,11 @@ static int find_plugger_helper_cb(char *fname, void *data)
 /* Find configuration file and read it into memory */
 static void do_read_config(void)
 {
-  if(first_handle) return;
+  if (first_handle) return;
+
   D("do_read_config\n");
   
-  if(!find_helper_file("mozpluggerrc-" VERSION,read_config_cb,0) &&
+  if (!find_helper_file("mozpluggerrc-" VERSION,read_config_cb,0) &&
      !find_helper_file("mozpluggerrc",read_config_cb,0))
   {
     fprintf(stderr,"MozPlugger: Unable to find mozpluggerrc file!\n");
@@ -457,10 +452,10 @@ static void do_read_config(void)
   D("do_read_config done\n");
 
   find_helper_file("mozplugger", find_plugger_helper_cb, 0);
-  if(inpath("mozplugger"))
+  if (inpath("mozplugger"))
     helper_fname="mozplugger";
 
-  if(!helper_fname)
+  if (!helper_fname)
     fprintf(stderr,"MozPlugger: Unable to find mozplugger-" VERSION "\n");
 }
 
@@ -479,22 +474,22 @@ char *NPP_GetMIMEDescription(void)
   D("Getmimedescription\n");
 
   size_required=0;
-  for(h=first_handle;h;h=h->next)
-    for(m=h->types;m;m=m->next)
+  for (h=first_handle;h;h=h->next)
+    for (m=h->types;m;m=m->next)
       size_required+=strlen(m->line)+1;
 
   D("Size required=%d\n",size_required);
   x=(char *)xalloc(size_required+1);
-  if(!x) return 0;
+  if (!x) return 0;
 
   D("Malloc did not fail\n");
 
   y=x;
 
-  for(h=first_handle;h;h=h->next)
+  for (h=first_handle;h;h=h->next)
   {
     D("Foo: %p\n",h->commands);
-    for(m=h->types;m;m=m->next)
+    for (m=h->types;m;m=m->next)
     {
 /*       D("appending: %s\n",m->line); */
       memcpy(y,m->line,strlen(m->line));
@@ -502,7 +497,7 @@ char *NPP_GetMIMEDescription(void)
       *(y++)=';';
     }
   }
-  if(x!=y) y--;
+  if (x!=y) y--;
   *(y++)=0;
   D("Getmimedescription done: %s\n",x);
   return x;
@@ -524,17 +519,17 @@ static int find_command(NPP instance,
 
   D("find_command...\n");
 
-  for(h=first_handle;h;h=h->next)
+  for (h=first_handle;h;h=h->next)
   {
     D("commands for this handle at (%p)\n",h->commands);
-    for(m=h->types;m;m=m->next)
+    for (m=h->types;m;m=m->next)
     {
       char *tmp1=FEND(m->line,':');
       char tmp2;
       int tmp3;
       D("Checking '%s'\n",m->line);
 
-      while(tmp1>m->line && isspace(tmp1[-1]))
+      while (tmp1>m->line && isspace(tmp1[-1]))
 	tmp1--;
 
       D("tmp1=%s\n",tmp1);
@@ -544,23 +539,26 @@ static int find_command(NPP instance,
       D("Checking '%s' ?= '%s'\n",m->line,THIS->mimetype);
       tmp3=strcasecmp(THIS->mimetype, m->line);
       *tmp1=tmp2;
-      if(!tmp3)
+      if (!tmp3)
       {
 	D("Match found!\n");
 	break;
-      }else{
+      } else {
 	D("Not same.\n");
       }
     }
 
-    if(m)
+    if (m)
     {
-      for(c=h->commands;c;c=c->next)
+      for (c=h->commands;c;c=c->next)
       {
 	D("Checking command: %s\n",c->cmd);
-	if((c->flags & H_LOOP) && THIS->repeats != MAXINT) continue;
-	if( (!!streaming) != (!!(c->flags & H_STREAM))) continue;
-	if(!inpath(c->cmd)) continue;
+	if ((c->flags & H_LOOP) && (THIS->repeats != MAXINT))
+	  continue;
+	if ((!!streaming) != (!!(c->flags & H_STREAM)))
+	  continue;
+	if (!inpath(c->cmd))
+	  continue;
 	D("Match found!\n");
 	THIS->command=c->cmd;
 	THIS->flags=c->flags;
@@ -630,22 +628,21 @@ NPError  NPP_New(NPMIMEType pluginType,
     return NPERR_INVALID_INSTANCE_ERROR;
 
   THIS = NPN_MemAlloc(sizeof(struct data));
-  if(!THIS) return NPERR_OUT_OF_MEMORY_ERROR;
+  if (!THIS) return NPERR_OUT_OF_MEMORY_ERROR;
   memset((char *)THIS, 0, sizeof(*THIS));
 
-  /* Change default to repeat once? */
-  THIS->repeats=0x7fffffff;
+  THIS->repeats=1;
   THIS->pid1=-1;
   THIS->pid2=-1;
   WINDOW=0;
   THIS->displayname=0;
   THIS->display=0;
   
-  if(!pluginType) return NPERR_GENERIC_ERROR;
+  if (!pluginType) return NPERR_GENERIC_ERROR;
 
-  for(e=0;e<argc;e++)
+  for (e=0;e<argc;e++)
   {
-    if(!strcasecmp("loop",argn[e]))
+    if (!strcasecmp("loop",argn[e]))
     {
       switch(argv[e][0])
       {
@@ -668,6 +665,8 @@ NPError  NPP_New(NPMIMEType pluginType,
 /* Free data, kill processes, it is time for this instance to die */
 NPError NPP_Destroy(NPP instance, NPSavedData** save)
 {
+  int status;
+
   if (!instance)
     return NPERR_INVALID_INSTANCE_ERROR;
 
@@ -675,13 +674,13 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
 
   if (THIS)
   {
-    if(THIS->pid1 > 0)
+    if (THIS->pid1 > 0)
     {
       kill(-THIS->pid1, SIGKILL);
       kill(THIS->pid1, SIGKILL);
     }
     
-    if(THIS->pid2 > 0)
+    if (THIS->pid2 > 0)
     {
       kill(-THIS->pid2, SIGKILL);
       kill(THIS->pid2, SIGKILL);
@@ -689,12 +688,17 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
 
     D("Freeing memory %p\n",THIS->mimetype);
     
-    if(THIS->mimetype)
+    if (THIS->mimetype)
     {
       NPN_MemFree(THIS->mimetype);
       THIS->mimetype=0;
     }
   }
+
+  D("Waiting sons\n");
+
+  while (waitpid(-1, &status, WNOHANG) > 0);
+
   D("Destroy finished\n");
   
   return NPERR_NO_ERROR;
@@ -717,35 +721,35 @@ NPError NPP_NewStream(NPP instance,
     return NPERR_INVALID_INSTANCE_ERROR;
 
   if (THIS->pid1 != -1)
-    return NPERR_NO_ERROR;
+    return NPERR_INVALID_INSTANCE_ERROR;
 
   /* This is a stupid special case and should be coded into
    * mozpluggerc instead...
    */
-  if(!strncasecmp("image/",type,6) ||
+  if (!strncasecmp("image/",type,6) ||
      !strncasecmp("x-image/",type,6))
     THIS->repeats=1;
 
   D("Mime type %s\n",type);
 
-  if(THIS->mimetype)
+  if (THIS->mimetype)
   {
     NPN_MemFree(THIS->mimetype);
     THIS->mimetype=0;
   }
   THIS->mimetype = NPN_MemAlloc(strlen(type)+1);
-  if(!THIS->mimetype) return NPERR_OUT_OF_MEMORY_ERROR;
+  if (!THIS->mimetype) return NPERR_OUT_OF_MEMORY_ERROR;
   strcpy(THIS->mimetype, type);
 
   D("Url is %s (seekable=%d)\n",stream->url,seekable);
   
-  if(!find_command(instance, 1) && !find_command(instance, 0))
+  if (!find_command(instance, 1) && !find_command(instance, 0))
     {
-      if(!inpath("xmessage"))
+      if (!inpath("xmessage"))
 	{
 	  NPN_Status(instance, "No appropriate application found !");
 	  return NPERR_GENERIC_ERROR;
-	}else{
+	} else {
 	  D("Using xmessage!!!\n");
 	  /* Hard-coded default */
 	  THIS->command="xmessage -buttons '' \"MozPlugger: No appropriate application for type $mimetype found!\"";
@@ -754,7 +758,7 @@ NPError NPP_NewStream(NPP instance,
 	}
     }
 
-  if((THIS->flags & H_STREAM)
+  if ((THIS->flags & H_STREAM)
      && strncasecmp(stream->url,"file:",5)
      && strncasecmp(stream->url,"imap:",5)
      && strncasecmp(stream->url,"mailbox:",8))
@@ -763,19 +767,19 @@ NPError NPP_NewStream(NPP instance,
     tempfile = 0;
 
     THIS->pid1=my_fork(instance);
-    if(THIS->pid1==-1)
+    if (THIS->pid1==-1)
     {
       NPN_Status(instance, "Streamer: My_Fork failed!");
       return NPERR_GENERIC_ERROR;
     }
     
-    if(!THIS->pid1)
+    if (!THIS->pid1)
     {
       D("CHILD RUNNING run() [1]\n");
       run(instance, stream->url);
       exit(69);
     } else {
-      if(THIS->repeats < MAXINT)
+      if (THIS->repeats < MAXINT)
 	THIS->repeats--;
     }
     
@@ -789,10 +793,8 @@ NPError NPP_NewStream(NPP instance,
 
 int32 NPP_WriteReady(NPP instance, NPStream *stream)
 {
-  if (!instance)
-    return 0;
-  if (!tempfile)
-    return 0;
+  if (!instance) return 0;
+  if (!tempfile) return 0;
   return 0x7fffffff;
 }
 
@@ -817,19 +819,19 @@ void NPP_StreamAsFile(NPP instance,
 {
   D("Streamasfile\n");
 
+  if (!instance) return;
   if (!tempfile) return;
   if (!fname) return;
-  if (instance == NULL) return;
 
   D("......going to fork......\n");
   THIS->pid2=my_fork(instance);
-  if(THIS->pid2==-1) return;
+  if (THIS->pid2==-1) return;
     
-  if(!THIS->pid2)
+  if (!THIS->pid2)
     {
       D("CHILD RUNNING run() [7]\n");
 
-      if(!find_command(instance, 1) && !find_command(instance, 0))
+      if (!find_command(instance, 1) && !find_command(instance, 0))
 	   _exit(EX_UNAVAILABLE);
 
       D("CHILD RUNNING run() [2]\n");
@@ -843,11 +845,13 @@ NPError NPP_Initialize(void)
   D("init\n");
   return NPERR_NO_ERROR;
 }
+
 jref NPP_GetJavaClass()
 {
   D("Getjavaclass\n");
   return NULL;
 }
+
 void NPP_Shutdown(void)
 {
   D("Shutdown\n");
@@ -885,12 +889,12 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
 
   XSync (THIS->display, FALSE);
 
-  if(THIS->pid1 != -1)
+  if (THIS->pid1 != -1)
   {
     D("Sending SIGWINCH to pid1 (%d)\n",THIS->pid1);
     kill(THIS->pid1, SIGWINCH);
   }
-  if(THIS->pid2 != -1)
+  if (THIS->pid2 != -1)
   {
     D("Sending SIGWINCH to pid2 (%d)\n",THIS->pid2);
     kill(THIS->pid2, SIGWINCH);
@@ -904,7 +908,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
 void NPP_Print(NPP instance, NPPrint* printInfo)
 {
   D("PRINT\n");
-  if(printInfo == NULL)
+  if (printInfo == NULL)
     return;
   
   if (instance != NULL)
