@@ -14,8 +14,6 @@
 #define H_MAXASPECT 0x400
 #define H_FILL 0x800
 
-#define BUFSIZE 1024*256
-
 #define my_timercmp(tvp, cmp, uvp) \
   ( (tvp)->tv_sec == (uvp)->tv_sec ? \
     (tvp)->tv_usec cmp (uvp)->tv_usec : \
@@ -33,13 +31,16 @@
 #define MINIMUM(X,Y) ((X)<(Y)?(X):(Y))
 
 #ifdef DEBUG
+#define DEBUG_FILENAME "/tmp/mozdebug"
 static FILE *debug_output=0;
 static FILE *getout()
 {
-  if(debug_output) return debug_output;
-  debug_output=fopen("/tmp/ndebug","a+");
+  if (debug_output)
+    return debug_output;
+  if (!(debug_output = fopen(DEBUG_FILENAME,"a+")))
+    return NULL;
   fprintf(debug_output,"------------\n");
-  return   debug_output;
+  return debug_output;
 }
 static void D(char *fmt, ...)
 {
